(function ($, drupalSettings) {
  'use strict';

  let countryId = 'edit-field-location-country';
  let cityId = 'edit-field-location-city-0-value';
  let latitudeId = 'edit-field-location-latitude-0-value';
  let longitudeId = 'edit-field-location-longitude-0-value';

  if (drupalSettings.cloud.country_id) {
    countryId = drupalSettings.cloud.country_id;
  }

  if (drupalSettings.cloud.city_id) {
    cityId = drupalSettings.cloud.city_id;
  }

  if (drupalSettings.cloud.latitude_id) {
    latitudeId = drupalSettings.cloud.latitude_id;
  }

  if (drupalSettings.cloud.longitude_id) {
    longitudeId = drupalSettings.cloud.longitude_id;
  }

  let setGeolocation = function () {

    if (!drupalSettings.cloud || !drupalSettings.cloud.geocoder_url) {
      return;
    }

    let country = $('#' + countryId).val();
    let city = $('#' + cityId).val();
    if (country === '_none' || city.trim().length === 0) {
      return;
    }
    let url = drupalSettings.cloud.geocoder_url;

    url = url.replace('country', country);
    url = url.replace('city', city);

    $.ajax({
      async: false,
      url: url,
      dataType: 'json',
      success: function success(data) {
        if (data && data.latitude && data.longitude) {
          let step = $('#' + latitudeId).attr('step');
          if (step && $.isNumeric(step)) {
            step = Number.parseFloat(step);
          }
          else {
            step = 0.000001;
          }
          step = 1 / step;
          let latitude = Math.round(data.latitude * step) / step;
          let longitude = Math.round(data.longitude * step) / step;
          $('#' + latitudeId).val(latitude);
          $('#' + longitudeId).val(longitude);
        }
      }
    });
  };

  $('#' + countryId).change(function () {
    setGeolocation();
  });

  $('#' + countryId).blur(function () {
    setGeolocation();
  });

  $('#' + cityId).change(function () {
    setGeolocation();
  });

  $('#' + cityId).blur(function () {
    setGeolocation();
  });

  $('form').submit(function () {
    setGeolocation();
  });

}(jQuery, drupalSettings));
