<?php

namespace Drupal\aws_cloud\Form\Vpc;

use Drupal\aws_cloud\Form\Ec2\AwsCloudContentForm;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Internet Gateway entity create form.
 *
 * @ingroup aws_cloud
 */
class InternetGatewayCreateForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {
    $form = parent::buildForm($form, $form_state);
    $this->ec2Service->setCloudContext($cloud_context);

    $weight = -50;

    $form['internet_gateway'] = [
      '#type' => 'details',
      '#title' => $this->t('Internet Gateway'),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['internet_gateway']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#required'      => TRUE,
    ];

    unset($form['tags']);

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);

    return $form;
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $cloud_context = $this->routeMatch->getParameter('cloud_context');

    $entity = $this->entity;
    $entity->setCloudContext($cloud_context);

    $result = $this->ec2Service->createInternetGateway([]);

    if (isset($result['InternetGateway'])
      && ($entity->setInternetGatewayId($result['InternetGateway']['InternetGatewayId']))
      && ($entity->save())
    ) {
      $this->updateNameAndCreatedByTags($entity, $entity->getInternetGatewayId());

      // Update the internet gateway.
      $this->ec2Service->updateInternetGateways([
        'InternetGatewayIds' => [$entity->getInternetGatewayId()],
      ], FALSE);

      $this->processOperationStatus($entity, 'created');
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSaveEvent($entity);
      $form_state->setRedirect(
        'view.aws_cloud_internet_gateway.list',
        ['cloud_context' => $entity->getCloudContext()]
      );
    }
    else {

      $this->processOperationErrorStatus($entity, 'created');
    }
  }

}
