<?php

namespace Drupal\aws_cloud\Form\Vpc;

use Drupal\aws_cloud\Form\Ec2\AwsCloudContentForm;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Internet Gateway entity edit forms.
 *
 * @ingroup aws_cloud
 */
class InternetGatewayEditForm extends AwsCloudContentForm {

  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;
    if (!$this->updateEntityBuildForm($entity)) {
      return $this->redirectUrl;
    }
    $this->ec2Service->setCloudContext($entity->getCloudContext());

    $weight = -50;

    $form['internet_gateway'] = [
      '#type' => 'details',
      '#title' => $this->t('Internet Gateway'),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['internet_gateway']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => TRUE,
    ];

    $form['internet_gateway']['internet_gateway_id'] = [
      '#type'          => 'item',
      '#title'         => $this->getItemTitle($this->t('Internet Gateway ID')),
      '#markup'        => $entity->getInternetGatewayId(),
    ];

    $form['internet_gateway']['state'] = [
      '#type'          => 'item',
      '#title'         => $this->getItemTitle($this->t('State')),
      '#markup'        => $entity->getState(),
    ];

    $form['internet_gateway']['vpc_id'] = [
      '#type'          => 'item',
      '#title'         => $this->getItemTitle($this->t('VPC ID')),
      '#markup'        => $entity->getVpcId(),
    ];

    $form['internet_gateway']['created'] = [
      '#type'          => 'item',
      '#title'         => $this->getItemTitle($this->t('Created')),
      '#markup'        => $this->dateFormatter->format($entity->created(), 'short'),
    ];

    $form['fieldset_tags'] = [
      '#type'          => 'details',
      '#title'         => $this->t('Tags'),
      '#open'          => TRUE,
      '#weight'        => $weight++,
    ];

    $form['fieldset_tags'][] = $form['tags'];
    unset($form['tags']);

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);
    $form['actions']['#weight'] = $weight++;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): void {
    // Call copyFormItemValues() to ensure the form array is intact.
    $this->copyFormItemValues($form);

    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;
    $old_internet_gateway = $this->entityTypeManager
      ->getStorage('aws_cloud_internet_gateway')
      ->load($entity->id());

    $this->ec2Service->setCloudContext($entity->getCloudContext());
    if (!empty($old_internet_gateway) && $entity->save()) {

      // Update tags.
      $tag_map = [];
      foreach ($entity->getTags() ?: [] as $tag) {
        $tag_map[$tag['item_key']] = $tag['item_value'];
      }
      $this->setTagsInAws($entity->getInternetGatewayId(), $tag_map);

      $this->updateNameAndCreatedByTags($entity, $entity->getInternetGatewayId());

      // Update the internet gateway.
      $this->ec2Service->updateInternetGateways([
        'InternetGatewayId' => $entity->getInternetGatewayId(),
      ], FALSE);

      $this->processOperationStatus($entity, 'updated');
      $this->clearCacheValues($entity->getCacheTags());
      $this->dispatchSaveEvent($entity);
    }
    else {
      $this->messenger->addError($this->t('Unable to update @label.', [
        '@label' => $entity->getEntityType()->getSingularLabel(),
      ]));
    }

    $form_state->setRedirect('view.aws_cloud_internet_gateway.list', [
      'cloud_context' => $entity->getCloudContext(),
    ]);
  }

}
