<?php

namespace Drupal\k8s\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\k8s\Traits\K8sFormTrait;

/**
 * Form controller for the Pod create form.
 *
 * @ingroup k8s
 */
class K8sPodCreateForm extends K8sCreateForm {

  use K8sFormTrait;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {
    $form = parent::buildForm($form, $form_state, $cloud_context);

    $form['time_scheduler'] = [
      '#type' => 'details',
      '#title' => $this->t('Time Scheduler'),
      '#open' => TRUE,
      '#weight' => $form['pod']['#weight'],
    ];

    $form['time_scheduler']['enable_time_scheduler'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable time scheduler'),
    ];

    $states = [
      'visible' => [
        ':input[name="enable_time_scheduler"]' => ['checked' => TRUE],
      ],
    ];

    $form['time_scheduler']['container'] = [
      '#type' => 'container',
      '#states' => $states,
    ];

    $form['time_scheduler']['container']['start_time_title'] = [
      '#type' => 'item',
      '#title' => 'Start-up Time',
    ];

    $hour_options = [];
    for ($i = 0; $i < 24; $i++) {
      $hour_options[$i] = sprintf('%02d', $i);
    }

    $minute_options = [];
    for ($i = 0; $i < 60; $i++) {
      $minute_options[$i] = sprintf('%02d', $i);
    }

    $form['time_scheduler']['container']['start_hour'] = [
      '#type' => 'select',
      '#options' => $hour_options,
      '#default_value' => 0,
      '#prefix' => '<div class= "container-inline">',
    ];

    $form['time_scheduler']['container']['start_minute'] = [
      '#type' => 'select',
      '#options' => $minute_options,
      '#default_value' => 0,
      '#prefix' => ': ',
      '#suffix' => '</div>',
    ];

    $form['time_scheduler']['container']['stop_time_title'] = [
      '#type' => 'item',
      '#title' => 'Stop Time',
    ];

    $form['time_scheduler']['container']['stop_hour'] = [
      '#type' => 'select',
      '#options' => $hour_options,
      '#default_value' => 6,
      '#prefix' => '<div class= "container-inline">',
    ];

    $form['time_scheduler']['container']['stop_minute'] = [
      '#type' => 'select',
      '#options' => $minute_options,
      '#default_value' => 0,
      '#prefix' => ': ',
      '#suffix' => '</div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *    Thrown when unable to create entity.
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    $this->k8sService->setCloudContext($cloud_context);

    $this->saveYamlsOfEntityDetail($form_state);

    $form_state->setRedirect(
      "view.k8s_{$this->getShortEntityTypeNameUnderscore($this->entity)}.list",
      ['cloud_context' => $this->entity->getCloudContext()]
    );
  }

}
